@@ -1,7 +1,7 @@
 package Data::Clean;
 
-our $DATE = '2016-12-29'; # DATE
-our $VERSION = '0.43'; # VERSION
+our $DATE = '2017-01-03'; # DATE
+our $VERSION = '0.44'; # VERSION
 
 use 5.010001;
 use strict;
@@ -17,8 +17,8 @@ use Scalar::Util qw();
     $main::fatpacked{"String/LineNumber.pm"} = '##line '.(1+__LINE__).' "'.__FILE__."\"\n".<<'STRING_LINENUMBER';
 #package String::LineNumber;
 #
-#our $DATE = '2014-12-10'; 
-#our $VERSION = '0.01'; 
+#our $DATE = '2014-12-10';
+#our $VERSION = '0.01';
 #
 #use 5.010001;
 #use strict;
@@ -60,8 +60,8 @@ STRING_LINENUMBER
     $main::fatpacked{"String/PerlQuote.pm"} = '##line '.(1+__LINE__).' "'.__FILE__."\"\n".<<'STRING_PERLQUOTE';
 #package String::PerlQuote;
 #
-#our $DATE = '2016-10-07'; 
-#our $VERSION = '0.02'; 
+#our $DATE = '2016-10-07';
+#our $VERSION = '0.02';
 #
 #use 5.010001;
 #use strict;
@@ -87,7 +87,7 @@ STRING_LINENUMBER
 #sub double_quote {
 #  local($_) = $_[0];
 #  s/([\\\"\@\$])/\\$1/g;
-#  return qq("$_") unless /[^\040-\176]/;  
+#  return qq("$_") unless /[^\040-\176]/;
 #
 #  s/([\a\b\t\n\f\r\e])/$esc{$1}/g;
 #
@@ -196,17 +196,49 @@ sub command_unbless_pp {
     "{{var}} = Function::Fallback::CoreOrPP::_unbless_fallback({{var}}); \$ref = ref({{var}})";
 }
 
-sub command_clone {
-    my $clone_func;
-    eval { require Data::Clone };
-    if ($@) {
-        require Clone::PP;
-        $clone_func = "Clone::PP::clone";
+sub command_unbless_ffc_inlined {
+    my ($self, $args) = @_;
+
+    # code taken from Function::Fallback::CoreOrPP 0.07
+    $self->{_subs}{unbless} //= <<'EOC';
+    my $ref = shift;
+
+    my $r = ref($ref);
+    # not a reference
+    return $ref unless $r;
+
+    # return if not a blessed ref
+    my ($r2, $r3) = "$ref" =~ /(.+)=(.+?)\(/
+        or return $ref;
+
+    if ($r3 eq 'HASH') {
+        return { %$ref };
+    } elsif ($r3 eq 'ARRAY') {
+        return [ @$ref ];
+    } elsif ($r3 eq 'SCALAR') {
+        return \( my $copy = ${$ref} );
     } else {
-        $clone_func = "Data::Clone::clone";
+        die "Can't handle $ref";
+    }
+EOC
+
+    "{{var}} = \$sub_unbless->({{var}}); \$ref = ref({{var}})";
+}
+
+sub command_clone {
+    my ($self, $args) = @_;
+
+    my $clone_func;
+    unless ($clone_func = $self->{opts}{'!clone_func'}) {
+        eval { require Data::Clone };
+        if ($@) {
+            require Clone::PP;
+            $clone_func = "Clone::PP::clone";
+        } else {
+            $clone_func = "Data::Clone::clone";
+        }
     }
 
-    my ($self, $args) = @_;
     my $limit = $args->[0] // 1;
     return join(
         "",
@@ -268,6 +300,8 @@ sub _generate_cleanser_code {
         $add_new_if->("\$ref eq '$ref'", $act0);
     };
 
+    $self->{_subs} = {};
+
     # catch circular references
     my $circ = $opts->{-circular};
     if ($circ) {
@@ -322,6 +356,9 @@ sub _generate_cleanser_code {
 
     push @code, 'sub {'."\n";
     push @code, 'require Scalar::Util;'."\n" if $opts->{'!recurse_obj'};
+    for (sort keys %{$self->{_subs}}) {
+        push @code, "state \$sub_$_ = sub { ".$self->{_subs}{$_}." };\n";
+    }
     push @code, 'my $data = shift;'."\n";
     push @code, 'state %refs;'."\n" if $circ;
     push @code, 'state $ctr_circ;'."\n" if $circ;
@@ -356,6 +393,8 @@ sub _generate_cleanser_code {
     }
     eval "\$self->{code} = $code";
     die "Can't generate code: $@" if $@;
+
+    delete $self->{_subs};
     $self->{src} = $code;
 }
 
@@ -387,7 +426,7 @@ Data::Clean - Clean data structure
 
 =head1 VERSION
 
-This document describes version 0.43 of Data::Clean (from Perl distribution Data-Clean), released on 2016-12-29.
+This document describes version 0.44 of Data::Clean (from Perl distribution Data-Clean), released on 2017-01-03.
 
 =head1 SYNOPSIS
 
@@ -447,13 +486,25 @@ arguments.
 Note that arrayrefs and hashrefs are always walked into, so it's not trapped by
 C<-ref>.
 
-Option keys that start with C<!> are special: C<!recurse_obj> (bool) which can
-be set to true to to recurse into objects if they are hash- or array-based. By
-default objects are not recursed into. Note that if you enable this option,
+Default for C<%opts>: C<< -ref => 'stringify' >>.
+
+Option keys that start with C<!> are special:
+
+=over
+
+=item * !recurse_obj (bool)
+
+Can be set to true to to recurse into objects if they are hash- or array-based.
+By default objects are not recursed into. Note that if you enable this option,
 object options (like C<Foo::Bar> or C<-obj>) won't work for hash- and
 array-based objects because they will be recursed instead.
 
-Default for C<%opts>: C<< -ref => 'stringify' >>.
+=item * !clone_func (str)
+
+Set name of clone function to use. The default is to use C<Data::Clone::clone>
+if available, or fallback to C<Clone::PP::clone>.
+
+=back
 
 Available commands:
 
@@ -588,7 +639,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2016 by perlancar@cpan.org.
+This software is copyright (c) 2017 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
